@@ -3,7 +3,7 @@ from contextlib import contextmanager
 from functools import partial
 from gc import get_referrers, get_referents
 from operator import setitem, getitem
-from types import ModuleType
+from types import ModuleType, MethodType
 from typing import Any, TypeVar, Callable, Dict, Tuple
 
 from testfixtures.resolve import resolve, not_there, Resolved, classmethod_type, class_type, Setter
@@ -66,7 +66,6 @@ class Replacer:
         if isinstance(target, str) and not name:
             resolved = resolve(target, container, sep)
         else:
-            found = not_there
             if container is None:
                 container = target
 
@@ -79,7 +78,7 @@ class Replacer:
                         accessor = getitem
                         found = accessor(container, name)
                     except KeyError:
-                        pass
+                        found = not_there
                     except TypeError:
                         accessor = getattr
                         found = accessor(container, name, not_there)
@@ -87,12 +86,19 @@ class Replacer:
                     try:
                         found = accessor(container, name)
                     except (KeyError, AttributeError):
-                        pass
+                        found = not_there
 
             if strict and not (found is not_there or target is container):
-                expected = accessor(container, name)
-                if target is not expected:
-                    raise AssertionError(f'{name!r} resolved to {found}, expected {target}')
+                if found is not target:
+                    if isinstance(found, MethodType):
+                        raise TypeError(
+                            'Cannot replace methods on instances with strict=True, '
+                            'replace on class or use strict=False'
+                        )
+                    raise AssertionError(
+                        f'{accessor} of {name!r} from {container!r} gave {found!r}, '
+                        f'expected {target}'
+                    )
 
             resolved = Resolved(
                 container,
@@ -103,9 +109,23 @@ class Replacer:
 
         if resolved.setter is None:
             raise ValueError('target must contain at least one dot!')
+
         if resolved.found is not_there and strict:
             raise AttributeError('Original %r not found' % resolved.name)
 
+        if (
+                hasattr(resolved.container, '__dict__') and
+                resolved.setter is setattr and
+                resolved.name not in resolved.container.__dict__
+        ):
+            if strict:
+                raise AttributeError(
+                    f'{resolved.container!r} has __dict__ but {resolved.name!r} is not in it'
+                )
+            else:
+                resolved.found = not_there
+
+
         replacement_to_use = replacement
 
         if isinstance(resolved.container, type):
@@ -148,7 +168,7 @@ class Replacer:
         for referrer in get_referrers(attribute):
             if break_on_static and isinstance(referrer, staticmethod):
                 return None, referrer
-            elif isinstance(referrer, dict) and '__dict__' in referrer:
+            elif isinstance(referrer, dict):
                 if referrer.get(name) is attribute:
                     for container in get_referrers(referrer):
                         if isinstance(container, type):
@@ -201,8 +221,8 @@ class Replacer:
         Restore all the original objects that have been replaced by
         calls to the :meth:`replace` method of this :class:`Replacer`.
         """
-        for id_, (target, original) in tuple(self.originals.items()):
-            self._replace(original, original.found)
+        for id_, (target, resolved) in tuple(self.originals.items()):
+            self._replace(resolved, resolved.found)
             del self.originals[id_]
 
     def __enter__(self):
